<?php

namespace Drupal\mollie_recurring_payments\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Datetime\DateFormatterInterface;
use Drupal\Core\Link;

/**
 * Class PaymentListBuilder.
 *
 * @package Drupal\mollie\Controller
 */
class RecurringPaymentListBuilder extends ControllerBase
{

  /**
   * Date formatter.
   *
   * @var \Drupal\Core\Datetime\DateFormatterInterface
   */
  protected $dateFormatter;

  /**
   * Constructs a new EntityListBuilder object.
   *
   * @param \Drupal\Core\Entity\EntityTypeInterface $entityType
   *   The entity type definition.
   * @param \Drupal\Core\Entity\EntityStorageInterface $storage
   *   The entity storage class.
   * @param \Drupal\Core\Datetime\DateFormatterInterface $dateFormatter
   *   Date formatter.
   */
  public function __construct(DateFormatterInterface $dateFormatter)
  {
    $this->dateFormatter = $dateFormatter;
  }

  /**
   * Build recurring payments list.
   */
  public function content()
  {
    return [
      '#type' => 'table',
      '#header' => $this->buildHeader(),
      '#rows' => $this->buildRow(),
      '#empty' => $this->t('No data available.'),
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildHeader()
  {
    $header = [];

    $header['description'] = $this->t('Description');
    $header['id'] = [
      'data' => $this->t('Payment ID'),
      'class' => [RESPONSIVE_PRIORITY_MEDIUM],
    ];
    $header['mode'] = [
      'data' => $this->t('Mode'),
      'class' => [RESPONSIVE_PRIORITY_MEDIUM],
    ];
    $header['recurring_payments'] = [
      'data' => $this->t('Recurring payment'),
      'class' => [RESPONSIVE_PRIORITY_MEDIUM],
    ];
    $header['amount'] = $this->t('Amount');
    $header['refunded_amount'] = [
      'data' => $this->t('Refunded amount'),
      'class' => [RESPONSIVE_PRIORITY_LOW],
    ];
    $header['refundable_amount'] = [
      'data' => $this->t('Refundable amount'),
      'class' => [RESPONSIVE_PRIORITY_LOW],
    ];
    $header['captured_amount'] = [
      'data' => $this->t('Captured mount'),
      'class' => [RESPONSIVE_PRIORITY_LOW],
    ];
    $header['charged_back_amount'] = [
      'data' => $this->t('Charged back amount'),
      'class' => [RESPONSIVE_PRIORITY_LOW],
    ];
    $header['status'] = $this->t('Status');
    $header['created'] = [
      'data' => $this->t('Created'),
      'class' => [RESPONSIVE_PRIORITY_LOW],
    ];
    $header['changed'] = [
      'data' => $this->t('Changed'),
      'class' => [RESPONSIVE_PRIORITY_LOW],
    ];

    // As we do not have operations on Mollie transactions, we do not need to
    // call the parent to include the operations column. This saves some space.
    return $header;
  }

  /**
   * {@inheritdoc}
   */
  public function buildRow()
  {
    $transaction_ids = \Drupal::entityQuery('mollie_payment')
      ->condition('customer_id', NULL, 'IS NOT NULL')
      ->range(0, 50)
      ->accessCheck(FALSE)
      ->execute();

    $transactions = $this->entityTypeManager()->getStorage('mollie_payment')->loadMultiple($transaction_ids);

    $rows = [];
    foreach ($transactions as $entity) {

      if (!isset($entity->get('customer_id')->value)) {
        continue;
      }

      $row = [];

      $row['description'] = Link::fromTextAndUrl($entity->getDescription(), $entity->toUrl());
      $row['id'] = $entity->id();
      $row['mode'] = $entity->getMode();

      $recurring_payment_data = [];
      if (isset($entity->get('customer_id')->value)) {

        // Customer data.
        $recurring_payment_data[] = 'customerID: ' . $entity->get('customer_id')->value;

        // Mandate data.
        $mollie = \Drupal::service('mollie.mollie');
        $mollie_client = $mollie->getClient();
        $customer_mandates = $mollie_client->customers->get($entity->get('customer_id')->value)->mandates();
        foreach ($customer_mandates as $mandate) {
          $recurring_payment_data[] = 'mandateID: ' . $mandate->id . '(' . $mandate->status . ')';
        }

        // Subscription data.
        if (isset($entity->get('subscription_id')->value)) {
          $recurring_payment_data[] = 'subscriptionID: ' . $entity->get('subscription_id')->value;
        }
      }
      $row['recurring_payments'] = implode(', ', $recurring_payment_data);

      $row['amount'] = $this->t(
        '@amount @currency',
        [
          '@amount' => $entity->getAmount(),
          '@currency' => $entity->getCurrency(),
        ]
      );
      $row['refunded_amount'] = $this->t(
        '@amount @currency',
        [
          '@amount' => $entity->getRefundedAmount(),
          '@currency' => $entity->getRefundedCurrency(),
        ]
      );
      $row['refundable_amount'] = $this->t(
        '@amount @currency',
        [
          '@amount' => $entity->getRefundableAmount(),
          '@currency' => $entity->getRefundableCurrency(),
        ]
      );
      $row['captured_amount'] = $this->t(
        '@amount @currency',
        [
          '@amount' => $entity->getCapturedAmount(),
          '@currency' => $entity->getCapturedCurrency(),
        ]
      );
      $row['charged_back_amount'] = $this->t(
        '@amount @currency',
        [
          '@amount' => $entity->getChargedBackAmount(),
          '@currency' => $entity->getChargedBackCurrency(),
        ]
      );
      $row['status'] = $entity->getStatus();
      $row['created'] = $this->getFormattedDate($entity->getCreatedTime());
      $row['changed'] = $this->getFormattedDate($entity->getChangedTime());

      $rows[] = $row;
    }
    return $rows;
  }

  /**
   * @param string $date
   *   Date in ISO 8601 format.
   *
   * @return string
   *   Date formatted in medium date format.
   */
  protected function getFormattedDate($date)
  {
    $dateTime = new \DateTime($date);
    return $this->dateFormatter
      ->format($dateTime->format('U'), 'medium');
  }
}
