<?php

namespace Drupal\mollie_recurring_payments\Entity;

use Drupal\mollie\Entity\PaymentStorage;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityStorageException;
use Drupal\Component\Serialization\Json;
use Drupal\Core\Url;
use Mollie\Api\Exceptions\ApiException;
use Mollie\Api\Resources\BaseResource;
use Drupal\mollie\Entity\Payment;
use Mollie\Api\Resources\Payment as MolliePayment;

/**
 * Class RecurringPaymentStorage.
 *
 * Overrides the Mollie PaymentStorage class to handle recurring payments.
 * https://docs.mollie.com/docs/recurring-payments#setting-up-the-first-payment.
 *
 * @package Drupal\mollie_recurring_payments\Entity
 */
class RecurringPaymentStorage extends PaymentStorage
{

  /**
   * {@inheritdoc}
   */
  protected function createEntityFromTransaction(BaseResource $payment): ?EntityInterface
  {
    if ($payment instanceof MolliePayment) {
      // If the metadata property is an object, we encode it into a JSON string
      // for consistency.
      if (is_object($payment->metadata)) {
        $payment->metadata = Json::encode($payment->metadata);
      }
      // If the metadata property is not an array already, we assume it is a
      // JSON string and decode it.
      $metadata = is_array($payment->metadata)
        ? $payment->metadata
        : Json::decode($payment->metadata ?? '{}');

      $values = [
        'id' => $payment->id,
        'created' => $payment->createdAt,
        'changed' => $this->getPaymentChangedDate($payment),
        'status' => $payment->status,
        'amount' => $payment->amount->value,
        'currency' => $payment->amount->currency,
        'refunded_amount' => $payment->amountRefunded ? $payment->amountRefunded->value : 0,
        'refunded_currency' => $payment->amountRefunded ? $payment->amountRefunded->currency : 'EUR',
        'refundable_amount' => $payment->amountRemaining ? $payment->amountRemaining->value : 0,
        'refundable_currency' => $payment->amountRemaining ? $payment->amountRemaining->currency : 'EUR',
        'captured_amount' => $payment->amountCaptured ? $payment->amountCaptured->value : 0,
        'captured_currency' => $payment->amountCaptured ? $payment->amountCaptured->currency : 'EUR',
        'charged_back_amount' => $payment->amountChargedBack ? $payment->amountChargedBack->value : 0,
        'charged_back_currency' => $payment->amountChargedBack ? $payment->amountChargedBack->value : 'EUR',
        'description' => $payment->description,
        'mode' => $payment->mode,
        'method' => $payment->method,
        'context' => $metadata['context'] ?? '',
        'context_id' => $metadata['context_id'] ?? '',
      ];

      // Custom fields for recurring payments.
      if (isset($payment->sequenceType)) {
        $values['sequence_type'] = $payment->sequenceType;
      }
      if (isset($payment->mandateId)) {
        $values['mandate_id'] = $payment->mandateId;
      }
      if (isset($payment->customerId)) {
        $values['customer_id'] = $payment->customerId;
      }
      if (isset($payment->subscriptionId)) {
        $values['subscription_id'] = $payment->subscriptionId;
      }

      if (isset($metadata['issuer'])) {
        $values['issuer'] = $metadata['issuer'];
      }
      if (isset($payment->_links->checkout)) {
        $values['checkout_url'] = $payment->_links->checkout->href;
      }

      return Payment::create($values);
    }

    return NULL;
  }

  /**
   * {@inheritdoc}
   */
  protected function createTransactionFromEntity(EntityInterface $entity): void
  {
    if ($entity instanceof Payment) {
      $redirectUrl = Url::fromRoute(
        'mollie.redirect',
        ['context' => $entity->getContext(), 'context_id' => $entity->getContextId()]
      )->setAbsolute()->toString();
      $webhookUrl = Url::fromRoute(
        'mollie.webhook.status_change',
        ['context' => $entity->getContext(), 'context_id' => $entity->getContextId()]
      )->setAbsolute()->toString();

      // Change webhook when using an alternative base URL in test mode.
      $config = $this->configFactory->get('mollie.config');
      if ($config->get('test_mode') && $config->get('webhook_base_url') !== '') {
        $defaultBaseUrl = Url::fromRoute('<front>')
          ->setAbsolute()->toString();
        $webhookUrl = str_replace(
          $defaultBaseUrl,
          "{$config->get('webhook_base_url')}/",
          $webhookUrl
        );
      }

      $metadata = [
        'context' => $entity->getContext(),
        'context_id' => $entity->getContextId(),
      ];

      $values = [
        'description' => $entity->getDescription(),
        'amount' => [
          'value' => number_format($entity->getAmount(), 2, '.', ''),
          'currency' => $entity->getCurrency(),
        ],
        'method' => $entity->getMethod(),
        'redirectUrl' => $redirectUrl,
        'webhookUrl' => $webhookUrl,
        'metadata' => Json::encode($metadata),
        'locale' => $this->getLocaleByCurrentContentLanguage(),
      ];

      if (in_array($entity->getMethod(), ['ideal']) && $entity->getIssuer()) {
        $values['issuer'] = $entity->getIssuer();
      }

      // Setting up the first payment for recurring payments.
      $customer_name = $entity->get('customer_name')->value ?? NULL;
      $customer_email = $entity->get('customer_email')->value ?? NULL;
      if ($customer_name && $customer_email) {

        // 1. Create a unique customer using the Customers API.
        $customer = $this->mollieApiClient->customers->create([
          'name' => $customer_name,
          'email' => $customer_email,
          'locale' => $this->getLocaleByCurrentContentLanguage(),
          'metadata' => Json::encode($metadata),
        ]);

        // 2. Save the customer’s ID data for the Mollie payment.
        $entity->set('customer_id', $customer->id);
        $entity->set('customer_name', $customer_name);
        $entity->set('customer_email', $customer_email);
        $entity->set('sequence_type', 'first');

        // 3. Create a payment for the customer by specifying the customerId and setting the sequenceType parameter to first.
        $values['customerId'] = $customer->id;
        $values['sequenceType'] = 'first';
      }

      try {
        // If the entity already has an ID, we assume it is an update event.
        if ($entity->id()) {
          // $payment = $this->mollieApiClient->payments->get($entity->id());
        } else {
          // Create a payment on the Mollie side.
          $payment = $this->mollieApiClient->payments->create($values);
          // Update the entity with the information added by Mollie.
          // @todo This might be incomplete. Look for a way to replace $entity.
          $entity->setOriginalId($payment->id);
          $entity->set('id', $payment->id);
          if (isset($payment->_links->checkout)) {
            $entity->set('checkout_url', $payment->_links->checkout->href);
          }
        }
      } catch (ApiException $e) {
        \Drupal::logger('mollie_recurring_payments')->error($e);
        throw new EntityStorageException('An error occurred while creating the payment.');
      }
    }
  }
}
