<?php

namespace Drupal\mollie_recurring_payments\EventSubscriber;

use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Drupal\mollie\Events\MollieTransactionStatusChangeEvent;
use Drupal\mollie\Entity\Payment;
use Drupal\Component\Serialization\Json;

/**
 * Class MollieRecurringPaymentsSubscriber.
 *
 * Charging periodically with subscriptions.
 * https://docs.mollie.com/docs/recurring-payments#charging-periodically-with-subscriptions.
 *
 * @package Drupal\mollie_recurring_payments
 */
class MollieRecurringPaymentsSubscriber implements EventSubscriberInterface
{

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents(): array
  {
    return [
      MollieTransactionStatusChangeEvent::EVENT_NAME => 'addSubscription',
    ];
  }

  /**
   * Create subscription for recurring payments from webform payments.
   *
   * @param \Drupal\mollie\Events\MollieTransactionStatusChangeEvent $event
   *   Event.
   */
  public function addSubscription(MollieTransactionStatusChangeEvent $event): void
  {
    // Return if the event context is not mollie_webform.
    if ($event->getContext() !== 'mollie_webform') {
      return;
    }

    // Get the transaction and check whether it is a payment.
    $transaction = $event->getTransaction();
    if (!($transaction instanceof Payment)) {
      return;
    }

    // Create subscription.
    if (isset($transaction->customer_id->value) && $transaction->sequence_type->value == 'first') {
      try {
        $mollie = \Drupal::service('mollie.mollie');
        $mollie_client = $mollie->getClient();

        // 1. Make sure the customer has a pending or valid mandate using the Mandates API.
        $mandates = $mollie_client->customers->get($transaction->customer_id->value)->mandates();

        // 2. Continue if there’s a mandate with its status being either pending or valid.
        foreach ($mandates as $mandate) {
          if ($mandate->status === 'pending' || $mandate->status === 'valid') {

            $metadata = [
              'context' => $transaction->getContext(),
              'context_id' => $transaction->getContextId(),
            ];

            // 3. Create the subscription.
            $subscription = $mollie_client->subscriptions->createForId($transaction->customer_id->value, [
              "amount" => [
                "currency" => $transaction->currency->value,
                "value" => $transaction->amount->value,
              ],
              "interval" => "12 months",
              "description" => t("Yearly subscription for ") . $transaction->description->value,
              "mandateId" => $mandate->id,
              "startDate" => date('Y-m-d', strtotime($transaction->created->value)),
              'metadata' => Json::encode($metadata),
            ]);

            // Save the subscription ID to the payment entity.
            $transaction->set('subscription_id', $subscription->id);
            $transaction->save();

            // Exit the loop after creating the subscription.
            break;
          }
        }
      } catch (\Exception $e) {
        \Drupal::logger('mollie_recurring_payments')->error($e);
      }
    }
  }
}
