<?php

namespace Drupal\mollie_recurring_payments\Plugin\WebformHandler;

use Drupal\Core\Entity\EntityStorageException;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Routing\TrustedRedirectResponse;
use Drupal\webform\Plugin\WebformHandlerBase;
use Drupal\webform\WebformSubmissionInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 *
 * @WebformHandler(
 *   id = "mollie_recurring_payment_handler",
 *   label = @Translation("Mollie recurring payment"),
 *   category = @Translation("Mollie"),
 *   description = @Translation("Creates a Mollie recurring payment from submitted form data."),
 *   cardinality = \Drupal\webform\Plugin\WebformHandlerInterface::CARDINALITY_SINGLE,
 * )
 */
class MollieRecurringPaymentHandler extends WebformHandlerBase
{

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The Mollie webform helper.
   *
   * @var \Drupal\mollie_webform\MollieWebformHelper
   */
  protected $mollieWebformHelper;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition)
  {
    $class = parent::create($container, $configuration, $plugin_id, $plugin_definition);
    $class->entityTypeManager = $container->get('entity_type.manager');
    $class->mollieWebformHelper = $container->get('mollie_webform.helper');
    return $class;
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state)
  {
    $form = parent::buildConfigurationForm($form, $form_state);

    $form['currency'] = [
      '#type' => 'select',
      '#title' => $this->t('Currency'),
      '#required' => TRUE,
      '#options' => ['EUR' => 'Euro', 'USD' => 'USD'],
      '#default_value' => $this->getConfiguration()['settings']['currency'] ?? '',
    ];

    // Build an options array for the elements on the webform.
    $elements = $this->getWebform()->getElementsDecodedAndFlattened();
    $options = [];

    foreach ($elements as $key => $element) {
      if (!empty($element['#title']) && isset($element['#type'])) {
        $options[$key] = $element['#title'] . " (" . $key . ")";
      }
    }

    $form['amount_element'] = [
      '#type' => 'select',
      '#title' => $this->t('Amount element'),
      '#description' => $this->t('Form element holding the amount for the payment.'),
      '#required' => TRUE,
      '#options' => $options,
      '#default_value' => $this->getConfiguration()['settings']['amount_element'] ?? '',
    ];

    $form['method_element'] = [
      '#type' => 'select',
      '#title' => $this->t('Payment method element'),
      '#description' => $this->t('Form element holding the payment method chosen in the webform. Leaving empty will allow choosing payment method in Mollie.'),
      '#required' => FALSE,
      '#options' => ['' => $this->t('None')] + $options,
      '#default_value' => $this->getConfiguration()['settings']['method_element'] ?? '',
    ];

    $form['description_element'] = [
      '#type' => 'select',
      '#title' => $this->t('Description element'),
      '#description' => $this->t('(optional) Form element holding the description for the payment. Defaults to "[form title] #[submission id]".'),
      '#options' => ['' => $this->t('None')] + $options,
      '#default_value' => $this->getConfiguration()['settings']['description_element'] ?? '',
    ];

    // Add fields for the Mollie customer.
    $form['customer'] = [
      '#type' => 'fieldgroup',
      '#title' => $this->t('Mollie customer data'),
    ];

    $form['customer']['customer_name'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Customer name'),
      '#description' => $this->t('Enter the name for the Mollie customer, [webform_submission:values:name] [webform_submission:values:lastname] or similar tokens can be combined.'),
      '#default_value' => $this->getConfiguration()['settings']['customer_name'] ?? '',
      '#required' => TRUE,
    ];

    // Build an options array for the emails on the webform.
    $form['customer']['customer_email'] = [
      '#type' => 'select',
      '#title' => $this->t('Customer Email'),
      '#description' => $this->t('Select the email for the Mollie customer.'),
      '#options' => $options,
      '#default_value' => $this->getConfiguration()['settings']['customer_email'] ?? '',
      '#required' => TRUE,
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state)
  {
    parent::submitConfigurationForm($form, $form_state);

    $this->configuration['currency'] = $form_state->getValue('currency');
    $this->configuration['amount_element'] = $form_state->getValue('amount_element');
    $this->configuration['description_element'] = $form_state->getValue('description_element');
    $this->configuration['method_element'] = $form_state->getValue('method_element');

    $this->configuration['customer_name'] = $form_state->getValue('customer')['customer_name'];
    $this->configuration['customer_email'] = $form_state->getValue('customer')['customer_email'];
  }

  /**
   * {@inheritdoc}
   */
  public function getSummary()
  {
    return [
      '#theme' => 'item_list',
      '#items' => [
        $this->t('Currency: %currency', ['%currency' => $this->getConfiguration()['settings']['currency']]),
        $this->t('Amount element: %amount_element', ['%amount_element' => $this->getConfiguration()['settings']['amount_element']]),
        $this->t('Description element: %description_element', ['%description_element' => $this->getConfiguration()['settings']['description_element']]),
        $this->t('Payment method element: %method_element', ['%method_element' => $this->getConfiguration()['settings']['method_element']]),
        $this->t('Customer name: %customer_name', ['%customer_name' => $this->getConfiguration()['settings']['customer_name']]),
        $this->t('Customer email element: %customer_email', ['%customer_email' => $this->getConfiguration()['settings']['customer_email']]),
      ],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function checkConditions(WebformSubmissionInterface $webform_submission)
  {
    $conditions_are_met = parent::checkConditions($webform_submission);

    // If not all conditions are met no further checks are need.
    if (!$conditions_are_met) {
      return $conditions_are_met;
    }

    // If other conditions are met, check that there is no payment already by
    // validating that there is no payment status set yet.
    $element_key = $this->mollieWebformHelper
      ->getMolliePaymentStatusElementNameFromWebformSubmission($webform_submission);
    return empty($webform_submission->getElementData($element_key));
  }

  /**
   * {@inheritdoc}
   */
  public function confirmForm(array &$form, FormStateInterface $form_state, WebformSubmissionInterface $webform_submission)
  {
    $description = $this->getWebform()->label() . ' #' . $webform_submission->id();
    if ($this->getConfiguration()['settings']['description_element']) {
      $submittedDescription = $webform_submission->getElementData($this->getConfiguration()['settings']['description_element']);
      $description = !empty($submittedDescription) ? $submittedDescription : $description;
    }

    // Replace tokens in the customer name.
    $customer_name = $this->getConfiguration()['settings']['customer_name'];
    $customer_name = $this->replaceTokens($customer_name, $webform_submission);

    $transaction = $this->entityTypeManager->getStorage('mollie_payment')->create(
      [
        'amount' => (float) $webform_submission->getElementData($this->getConfiguration()['settings']['amount_element']),
        'currency' => $this->getConfiguration()['settings']['currency'],
        'description' => $description,
        'context' => 'mollie_webform',
        'context_id' => $webform_submission->id(),
        'method' => $webform_submission->getElementData($this->getConfiguration()['settings']['method_element']) ?? [],
        'customer_name' => $customer_name ?? '',
        'customer_email' => $webform_submission->getElementData($this->getConfiguration()['settings']['customer_email']) ?? '',
      ]
    );

    try {
      // Create the Mollie payment.
      $transaction->save();

      // Redirect to Mollie.
      $response = new TrustedRedirectResponse($transaction->getCheckoutUrl(), '303');
      $form_state->setResponse($response);
    } catch (EntityStorageException $e) {
      \Drupal::logger('mollie_recurring_payments')->error($e);
    }
  }
}
